@@ -155,6 +155,7 @@ int SubstItem::parseServerVar(const char *pCurLine,
 {
     const char *pName = pFormatStr;
     const char *pClose = findEndOfVarName(pName, pEnd);
+
     if (*pName == '{')
     {
         if (pClose == pEnd)
@@ -660,11 +661,15 @@ int RequestVars::getReqVar(HttpSession *pSession, int type, char *&pValue,
             p += 7;
         }
         i = pReq->getHeaderLen(HttpHeader::H_HOST);
+        if (i > pValue + bufLen - p)
+            i = pValue + bufLen - p;
         memmove(p, pReq->getHeader(HttpHeader::H_HOST),
                 i);
         p += i;
 
         i = pReq->getOrgURILen();
+        if (i > pValue + bufLen - p)
+            i = pValue + bufLen - p;
         memmove(p, pReq->getOrgURI(), i);
         p += i;
         return p - pValue;
